/*                                                                            */

/*----------------------------------------------------------------------------*/
/* System includes                                                            */
/*----------------------------------------------------------------------------*/

	#include <clib/alib_protos.h>

	#include <proto/exec.h>
	#include <proto/dos.h>
	#include <proto/graphics.h>
	#include <proto/intuition.h>

	#define ALL_REACTION_CLASSES
	#define __NOLIBBASE__
	#include <reaction/reaction.h>

/*----------------------------------------------------------------------------*/
/* Constants and macros                                                       */
/*----------------------------------------------------------------------------*/

	#define GID_END 99

/*----------------------------------------------------------------------------*/
/* Type definitions                                                           */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Global variables                                                          */
/*----------------------------------------------------------------------------*/

	struct ClassLibrary *WindowBase; Class *WindowClass;
	struct ClassLibrary *LayoutBase; Class *LayoutClass;
	struct ClassLibrary *ButtonBase; Class *ButtonClass;
	struct ClassLibrary *ListBrowserBase; Class *ListBrowserClass;

/*----------------------------------------------------------------------------*/
/* OS4 functionality for OS3: open a class library                            */
/*----------------------------------------------------------------------------*/

#ifndef __amigaos4__
	struct ClassLibrary *OpenClass (STRPTR name, ULONG version, Class **cl_ptr)

	{
	struct Library *LayoutBase;

	if (LayoutBase = OpenLibrary (name, version))
		{
		*cl_ptr = LAYOUT_GetClass();
		}

	return ((struct ClassLibrary *)LayoutBase);
	}

/*----------------------------------------------------------------------------*/
/* OS4 functionality for OS3: close a class library                           */
/*----------------------------------------------------------------------------*/

	VOID CloseClass (struct ClassLibrary *cl)

	{
	CloseLibrary (&cl->cl_Lib);
	}

#endif
/*----------------------------------------------------------------------------*/
/* Open all required class libraries                                          */
/*----------------------------------------------------------------------------*/

	BOOL open_libs (void)

	{
	WindowBase = OpenClass ("window.class", 0, &WindowClass);
	LayoutBase = OpenClass ("gadgets/layout.gadget", 0, &LayoutClass);
	ButtonBase = OpenClass ("gadgets/button.gadget", 0, &ButtonClass);
	ListBrowserBase = OpenClass ("gadgets/listbrowser.gadget", 0, &ListBrowserClass);

	return (WindowBase && LayoutBase && ButtonBase && ListBrowserClass);
	}

/*----------------------------------------------------------------------------*/
/* Close all opened class libraries                                           */
/*----------------------------------------------------------------------------*/

	void close_libs (void)

	{
	if (ListBrowserBase) CloseClass (ListBrowserBase);
	if (ButtonBase) CloseClass (ButtonBase);
	if (LayoutBase) CloseClass (LayoutBase);
	if (WindowBase) CloseClass (WindowBase);
	}

/*----------------------------------------------------------------------------*/
/* Main programm                                                              */
/*----------------------------------------------------------------------------*/

	int main (void)

	{
	struct Screen *scr;

	if (!open_libs())
		{
		Printf ("cannot open required ReAction classes\n");
		close_libs();
		return (RETURN_FAIL);
		}

	if ((scr = LockPubScreen (NULL)))
		{
		Object *winobj;
		struct ColumnInfo ci[] = {
			{100, NULL, 0},
			{100, NULL, 0},
			{100, NULL, 0},
			{-1, NULL, 0},
			};
		struct List labels;
		struct Node *node;

		NewList (&labels);

		if (node = AllocListBrowserNode (3,
				LBNA_Flags, LBFLG_CUSTOMPENS,
				LBNA_Column, 0, LBNCA_FGPen,  5, LBNCA_Text, "One",
				LBNA_Column, 1, LBNCA_FGPen,  6, LBNCA_Text, "Two",
				LBNA_Column, 2, /*LBNCA_FGPen,  7,*/ LBNCA_Text, "Three",
				TAG_END))
			AddTail (&labels, node);

		if (node = AllocListBrowserNode (3,
				LBNA_Flags, LBFLG_CUSTOMPENS,
				LBNA_Column, 0, LBNCA_FGPen,  9, LBNCA_Text, "Four",
				LBNA_Column, 1, /*LBNCA_FGPen, 10,*/ LBNCA_Text, "Five",
				LBNA_Column, 2, LBNCA_FGPen, 11, LBNCA_Text, "Six",
				TAG_END))
			AddTail (&labels, node);

		if (node = AllocListBrowserNode (3,
				LBNA_Flags, LBFLG_CUSTOMPENS,
				LBNA_Column, 0, /*LBNCA_FGPen, 13,*/ LBNCA_Text, "Seven",
				LBNA_Column, 1, LBNCA_FGPen, 14, LBNCA_Text, "Eight",
				LBNA_Column, 2, LBNCA_FGPen, 15, LBNCA_Text, "Nine",
				TAG_END))
			AddTail (&labels, node);

		if ((winobj = NewObject (WindowClass, NULL,
				WA_PubScreen, scr,
				WA_Title, "ReAct",
				WA_Flags, WFLG_CLOSEGADGET|WFLG_DRAGBAR|WFLG_DEPTHGADGET|WFLG_SIZEBBOTTOM|WFLG_SIZEGADGET|WFLG_ACTIVATE,
				WA_IDCMP, IDCMP_VANILLAKEY,
				WA_Width, scr->Width / 3,
				WA_Height, scr->Height / 4,
				WINDOW_Position, WPOS_CENTERSCREEN,
				WINDOW_ParentGroup, NewObject (LayoutClass, NULL,
					LAYOUT_Orientation, LAYOUT_ORIENT_VERT,
					LAYOUT_SpaceOuter, TRUE,
					LAYOUT_AddChild, NewObject(ListBrowserClass, NULL,
						LISTBROWSER_ColumnInfo, ci,
						LISTBROWSER_Labels, &labels,
						TAG_END),
					LAYOUT_AddChild, NewObject (LayoutClass, NULL,
						LAYOUT_FixedVert, FALSE,
						LAYOUT_EvenSize, TRUE,
						LAYOUT_AddChild, NewObject (ButtonClass, NULL,
							GA_Text, "Continue",
							TAG_END),
						LAYOUT_AddChild, NewObject(ButtonClass, NULL,
							GA_Text, "Quit",
							GA_ID, GID_END,
							GA_RelVerify, TRUE,
							TAG_END),
						TAG_END),
					TAG_END),
				TAG_END)))
			{
			struct Window *win;

			if ((win = (struct Window *) DoMethod (winobj, WM_OPEN, NULL)))
				{
				ULONG winsig;
				ULONG result;
				BOOL cont = TRUE;

				GetAttr (WINDOW_SigMask, winobj, &winsig);

				do	{
					if (Wait (winsig | SIGBREAKF_CTRL_C) & SIGBREAKF_CTRL_C)
						cont = FALSE;

					while ((result = DoMethod (winobj, WM_HANDLEINPUT, NULL)) != WMHI_LASTMSG)
						{
						switch (result & WMHI_CLASSMASK)
							{
						case WMHI_GADGETUP:
							switch (result & WMHI_GADGETMASK)
								{
							case GID_END:
								cont = FALSE;
								break;
								}
							break;
						case WMHI_VANILLAKEY:
							if ((result & WMHI_KEYMASK) == 0x1b)
								cont = FALSE;
							break;
						case WMHI_CLOSEWINDOW:
							cont = FALSE;
							break;
						default:
							break;
							}
						}
					}
				while (cont);
				}

			DisposeObject (winobj);
			}

		UnlockPubScreen (NULL, scr);
		}

	close_libs();

	return (RETURN_OK);
	}

/*----------------------------------------------------------------------------*/
/* End of source text                                                         */
/*----------------------------------------------------------------------------*/
